package ch.psi.daq.test.queryrest.query;

import java.math.BigDecimal;
import java.util.List;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import java.util.regex.Pattern;
import java.util.stream.LongStream;
import java.util.stream.Stream;

import com.google.common.collect.Lists;

import ch.psi.daq.common.ordering.Ordering;
import ch.psi.daq.common.time.TimeUtils;
import ch.psi.daq.domain.DataEvent;
import ch.psi.daq.domain.cassandra.ChannelEvent;
import ch.psi.daq.domain.reader.Backend;
import ch.psi.daq.domain.reader.DataReader;

public class DummyArchiverApplianceReader implements DataReader {
   public static final String ARCHIVER_TEST_CHANNEL = "ArchiverTestChannel_";

   public static final String TEST_CHANNEL_1 = "ArchiverChannel_1";
   public static final String TEST_CHANNEL_2 = "ArchiverChannel_2";
   private List<String> channels = Lists.newArrayList(TEST_CHANNEL_1, TEST_CHANNEL_2);

   private final Random random = new Random(0);
   private AtomicLong channelNameCallCounter = new AtomicLong();


   @Override
   public Backend getBackend() {
      return Backend.archiverappliance;
   }

   @Override
   public Stream<String> getChannelStream(String regex) {
      channels.add(ARCHIVER_TEST_CHANNEL + channelNameCallCounter.incrementAndGet());

      Stream<String> channelStream = channels.stream();
      if (regex != null) {
         Pattern pattern = Pattern.compile(regex);
         channelStream = channelStream.filter(channel -> pattern.matcher(channel).find());
      }

      return channelStream;
   }

   @Override
   public Stream<? extends DataEvent> getEventStream(String channel, long startPulseId, long endPulseId,
         Ordering ordering, boolean aggregateValues, String... columns) {
      return getElements(channel, startPulseId, endPulseId);
   }

   @Override
   public Stream<? extends DataEvent> getEventStream(String channel, BigDecimal startTime, BigDecimal endTime,
         Ordering ordering, boolean aggregateValues, String... columns) {
      return getElements(channel, TimeUtils.getMillis(startTime) / 10, TimeUtils.getMillis(endTime) / 10);
   }

   protected Stream<? extends DataEvent> getElements(String channel, long start, long end) {
      String channelLower = channel.toLowerCase();

      Stream<? extends DataEvent> eventStream = LongStream.rangeClosed(start, end).mapToObj(i -> {
         BigDecimal time = TimeUtils.getTimeFromMillis(i * 10, 0);

         if (channelLower.contains("waveform")) {
            long[] value = random.longs(2048).toArray();
            value[0] = i;
            return new ChannelEvent(
                  channel,
                  time,
                  i,
                  time,
                  1,
                  value
            );

         } else if (channelLower.contains("image")) {
            int x = 640;
            int y = 480;
            int[] shape = new int[] {x, y};
            long[] value = random.longs(x * y).toArray();
            value[0] = i;
            return new ChannelEvent(
                  channel,
                  time,
                  i,
                  time,
                  1,
                  value,
                  shape
            );
         } else {
            return new ChannelEvent(
                  channel,
                  time,
                  i,
                  time,
                  1,
                  i
            );
         }
      });

      return eventStream;
   }
}
